==> bootstrap_like_breakpoints-0.0.1.tar.gz: lib/bootstrap_like_breakpoints.dart <==
library bootstrap_like_breakpoints;

/*
  Bradley Honeyman
  Oct 27, 2021

  Ref:
  https://getbootstrap.com/docs/5.0/layout/containers/

 */

import 'package:flutter/material.dart';

/// A class for managing break points, their labels and values
/// values are based on the current screen width
class BSBreakPoints {
  /// a list containing the breakpoint labels from smallest to largest
  /// "" -> xxl
  static final List<String> _breakPointLabels = <String>[
    "",
    "sm",
    "md",
    "lg",
    "xl",
    "xxl",
  ];

  /// Map containing breakpoint labels as keys, and the
  /// corresponding breakpoint widths
  /// "": 576
  /// sm: 576
  /// md: 768
  /// lg: 992
  /// xl: 1200
  /// xxl: 1400
  static final Map<String, double> _breakPoints = {
    _breakPointLabels[0]: 576, //<
    _breakPointLabels[1]: 576, //>=
    _breakPointLabels[2]: 768, //>=
    _breakPointLabels[3]: 992, //>=
    _breakPointLabels[4]: 1200, //>=
    _breakPointLabels[5]: 1400, //>=
  };

  /// Map containing breakpoint labels, and the corresponding
  /// container widths, note that "" breakpoint is -1.
  /// this is because is must be set to screen width
  /// "": screen width
  /// sm: 540
  /// md: 720
  /// lg: 960
  /// xl: 1140
  /// xxl: 1320
  static final Map<String, double> _containerWidths = {
    _breakPointLabels[0]: -1, // set to full width
    _breakPointLabels[1]: 540,
    _breakPointLabels[2]: 720,
    _breakPointLabels[3]: 960,
    _breakPointLabels[4]: 1140,
    _breakPointLabels[5]: 1320,
  };

  /// returns map with breakpoints
  ///     "": 576, //<
  ///     "sm": 576, //>=
  ///     "md": 768, //>=
  ///     "lg": 992, //>=
  ///     "xl": 1140, //>=
  ///     "xxl": 1400, //>=
  static Map<String, double> getBreakPoints() {
    return _breakPoints;
  }

  /// returns a container width based on the passed breakpoint
  /// "": -1, because must be generated
  /// sm: 540,
  /// md: 720,
  /// lg: 960,
  /// xl: 1140,
  /// xxl: 1320,
  static double getContainerWidth(String key) {
    if (!_breakPointLabels.contains(key)) {
      throw FormatException(
          "BSBreakpoint: Passed key '$key' is invalid, must be:\n${_breakPointLabels.toString()}");
    }
    return _containerWidths[key]!;
  }

  /// returns all container widths, based on breakpoint
  /// "": -1, because must be generated
  /// sm: 540,
  /// md: 720,
  /// lg: 960,
  /// xl: 1140,
  /// xxl: 1320,
  static Map<String, double> getContainerWidths() {
    return _containerWidths;
  }

  /// returns array of break point labels
  /// ""
  /// "sm"
  /// "md"
  /// "lg"
  /// "xl"
  /// "xxl"
  static List<String> getBreakPointLabels() {
    return _breakPointLabels;
  }

  static dynamic valueBasedOnBreakPointFromDefinedWidth({
    required double width,
    required Map<String, dynamic> map,
  }) {
    // confirm keys count
    if (_breakPointLabels.length != map.keys.length) {
      throw FormatException(
          "BSBreakPoints:\nMust pass exactly the breakpoint labels as keys:\n${_breakPointLabels.toString()}:\nYou passed:\n${map.keys.toString()}");
    }

    // confirm that all keys passed are valid
    for (int i = 0; i < _breakPointLabels.length; i++) {
      if (!_breakPointLabels.contains(map.keys.elementAt(i))) {
        throw FormatException(
            "BSBreakPoints:\nAll keys passed must be valid. Valid values are (must be lowercase):\n${_breakPointLabels.toString()}:\nPassed:\n${map.keys.toString()}");
      }
    }

    // check from largest to smallest, which to return
    for (int i = _breakPointLabels.length - 1; i > 0; i--) {
      if (width >= _breakPoints[_breakPointLabels[i]]!) {
        return map[_breakPointLabels[i]];
      }
    }
    return map[_breakPointLabels[0]];
  }

  /// returns a custom value based on the current screen size
  /// valid key values are (case sensitive, only lowercase)
  /// ''
  /// 'sm'
  /// 'md'
  /// 'lg'
  /// 'xl'
  /// 'xxl'
  static dynamic valueBasedOnBreakPoint({
    required BuildContext context,
    required Map<String, dynamic> map,
  }) {
    // get screen width
    double screenWidth = MediaQuery.of(context).size.width;
    return valueBasedOnBreakPointFromDefinedWidth(
      width: screenWidth,
      map: map,
    );
  }

  /// returns the last 2, or only last char of a string as a double
  /// only one if the 2nd last isn't a char
  static double _getSuffixNumbersValue(String value) {
    int lastIndex = value.length - 1;
    if (double.tryParse(value[lastIndex - 1]) != null) {
      return double.parse(value[lastIndex - 1] + value[lastIndex]);
    } else {
      return double.parse(value[lastIndex]);
    }
  }

  /// returns the current breakpoint label based on the screen width
  static String currentBreakPointLabel(BuildContext context) {
    // get screen width
    double screenWidth = MediaQuery.of(context).size.width;

    // check from largest to smallest
    for (int i = _breakPointLabels.length - 1; i > 0; i--) {
      if (screenWidth >= _breakPoints[_breakPointLabels[i]]!) {
        return _breakPointLabels[i];
      }
    }
    return _breakPointLabels[0];
  }

  /// returns the width a column should be based on it's breakpoints
  /// ex: col-lg-6 would set width to 50% of container width when breakpoint is lg
  static double columnWidthFromBreakPointTriggers({
    required double maxWidth,
    required double screenWidth,
    required List<String> breakPointTriggers,
  }) {
    // set values to unset
    Map<String, double> map = <String, double>{
      "": maxWidth,
      "sm": -1,
      "md": -1,
      "lg": -1,
      "xl": -1,
      "xxl": -1,
    };

    // set explicit values
    for (int i = 0; i < breakPointTriggers.length; i++) {
      if (RegExp(r"^col$").firstMatch(breakPointTriggers[i]) != null) {
        map[""] = (1 / 12) * maxWidth;
      } else if (RegExp(r"^col-(([0-9])|([0-9][0-2]))$")
              .firstMatch(breakPointTriggers[i]) !=
          null) {
        map[""] =
            (_getSuffixNumbersValue(breakPointTriggers[i]) / 12) * maxWidth;
      } else {
        for (int r = 1; r < _breakPointLabels.length; r++) {
          if (RegExp("^col-${_breakPointLabels[r]}-(([0-9])|([0-9][0-2]))\$")
                  .firstMatch(breakPointTriggers[i]) !=
              null) {
            map[_breakPointLabels[r]] =
                (_getSuffixNumbersValue(breakPointTriggers[i]) / 12) * maxWidth;
          }
        }
      }
    }

    // set inferred values
    for (int i = 1; i < _breakPointLabels.length; i++) {
      if (map[_breakPointLabels[i]] == -1) {
        map[_breakPointLabels[i]] = map[_breakPointLabels[i - 1]]!;
      }
    }

    return valueBasedOnBreakPointFromDefinedWidth(width: screenWidth, map: map);
  }
}

==> bootstrap_like_breakpoints-0.1.0.tar.gz: lib/bootstrap_like_breakpoints.dart <==
library bootstrap_like_breakpoints;

/*
  Bradley Honeyman
  Oct 27, 2021

  Ref:
  https://getbootstrap.com/docs/5.0/layout/containers/

 */

import 'package:flutter/material.dart';

/// A class for managing break points, their labels and values
/// values are based on the current screen width
class BSBreakPoints {
  /// a list containing the breakpoint labels from smallest to largest
  /// "" -> xxl
  static final List<String> _breakPointLabels = <String>[
    "",
    "sm",
    "md",
    "lg",
    "xl",
    "xxl",
  ];

  /// Map containing breakpoint labels as keys, and the
  /// corresponding breakpoint widths
  /// "": 576
  /// sm: 576
  /// md: 768
  /// lg: 992
  /// xl: 1200
  /// xxl: 1400
  static final Map<String, double> _breakPoints = {
    _breakPointLabels[0]: 576, //<
    _breakPointLabels[1]: 576, //>=
    _breakPointLabels[2]: 768, //>=
    _breakPointLabels[3]: 992, //>=
    _breakPointLabels[4]: 1200, //>=
    _breakPointLabels[5]: 1400, //>=
  };

  /// Map containing breakpoint labels, and the corresponding
  /// container widths, note that "" breakpoint is -1.
  /// this is because is must be set to screen width
  /// "": screen width
  /// sm: 540
  /// md: 720
  /// lg: 960
  /// xl: 1140
  /// xxl: 1320
  static final Map<String, double> _containerWidths = {
    _breakPointLabels[0]: -1, // set to full width
    _breakPointLabels[1]: 540,
    _breakPointLabels[2]: 720,
    _breakPointLabels[3]: 960,
    _breakPointLabels[4]: 1140,
    _breakPointLabels[5]: 1320,
  };

  /// List containing types of text
  /// p
  /// h1
  /// h2
  /// h3
  /// h4
  /// h5
  /// h6
  static final List<String> _textTypes = [
    "p",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
  ];

  /// A map containing the small and large text sizes for the text that
  /// are a _textTypes
  /// p
  /// h1
  /// h2
  /// h3
  /// h4
  /// h5
  /// h6
  static final Map<String, Map<String, double>> _textTypesAndSizes = {
    _textTypes[0]: <String, double>{
      "sm": 16,
      "lg": 16,
    },
    _textTypes[1]: <String, double>{
      "sm": 32,
      "lg": 40,
    },
    _textTypes[2]: <String, double>{
      "sm": 25,
      "lg": 32,
    },
    _textTypes[3]: <String, double>{
      "sm": 22,
      "lg": 28,
    },
    _textTypes[4]: <String, double>{
      "sm": 24,
      "lg": 24,
    },
    _textTypes[5]: <String, double>{
      "sm": 16,
      "lg": 20,
    },
    _textTypes[6]: <String, double>{
      "sm": 13,
      "lg": 16,
    },
  };

  /// returns map with breakpoints
  ///     "": 576, //<
  ///     "sm": 576, //>=
  ///     "md": 768, //>=
  ///     "lg": 992, //>=
  ///     "xl": 1140, //>=
  ///     "xxl": 1400, //>=
  static Map<String, double> getBreakPoints() {
    return _breakPoints;
  }

  /// returns a container width based on the passed breakpoint
  /// "": -1, because must be generated
  /// sm: 540,
  /// md: 720,
  /// lg: 960,
  /// xl: 1140,
  /// xxl: 1320,
  static double getContainerWidth(String key) {
    if (!_breakPointLabels.contains(key)) {
      throw FormatException(
          "BSBreakpoint:\nPassed key '$key' is invalid, must be:\n${_breakPointLabels.toString()}");
    }
    return _containerWidths[key]!;
  }

  /// returns all container widths, based on breakpoint
  /// "": -1, because must be generated
  /// sm: 540,
  /// md: 720,
  /// lg: 960,
  /// xl: 1140,
  /// xxl: 1320,
  static Map<String, double> getContainerWidths() {
    return _containerWidths;
  }

  /// returns array of break point labels
  /// ""
  /// "sm"
  /// "md"
  /// "lg"
  /// "xl"
  /// "xxl"
  static List<String> getBreakPointLabels() {
    return _breakPointLabels;
  }

  static dynamic valueBasedOnBreakPointFromDefinedWidth({
    required double width,
    required Map<String, dynamic> map,
  }) {
    // confirm keys count
    if (_breakPointLabels.length != map.keys.length) {
      throw FormatException(
          "BSBreakPoints:\nMust pass exactly the breakpoint labels as keys:\n${_breakPointLabels.toString()}:\nYou passed:\n${map.keys.toString()}");
    }

    // confirm that all keys passed are valid
    for (int i = 0; i < _breakPointLabels.length; i++) {
      if (!_breakPointLabels.contains(map.keys.elementAt(i))) {
        throw FormatException(
            "BSBreakPoints:\nAll keys passed must be valid. Valid values are (must be lowercase):\n${_breakPointLabels.toString()}:\nPassed:\n${map.keys.toString()}");
      }
    }

    // check from largest to smallest, which to return
    for (int i = _breakPointLabels.length - 1; i > 0; i--) {
      if (width >= _breakPoints[_breakPointLabels[i]]!) {
        return map[_breakPointLabels[i]];
      }
    }
    return map[_breakPointLabels[0]];
  }

  /// returns a custom value based on the current screen size
  /// valid key values are (case sensitive, only lowercase)
  /// ''
  /// 'sm'
  /// 'md'
  /// 'lg'
  /// 'xl'
  /// 'xxl'
  static dynamic valueBasedOnBreakPoint({
    required BuildContext context,
    required Map<String, dynamic> map,
  }) {
    // get screen width
    double screenWidth = MediaQuery.of(context).size.width;
    return valueBasedOnBreakPointFromDefinedWidth(
      width: screenWidth,
      map: map,
    );
  }

  /// returns the last 2, or only last char of a string as a double
  /// only one if the 2nd last isn't a char
  static double _getSuffixNumbersValue(String value) {
    int lastIndex = value.length - 1;
    if (double.tryParse(value[lastIndex - 1]) != null) {
      return double.parse(value[lastIndex - 1] + value[lastIndex]);
    } else {
      return double.parse(value[lastIndex]);
    }
  }

  /// returns the current breakpoint label based on the screen width
  static String currentBreakPointLabel(BuildContext context) {
    // get screen width
    double screenWidth = MediaQuery.of(context).size.width;

    // check from largest to smallest
    for (int i = _breakPointLabels.length - 1; i > 0; i--) {
      if (screenWidth >= _breakPoints[_breakPointLabels[i]]!) {
        return _breakPointLabels[i];
      }
    }
    return _breakPointLabels[0];
  }

  /// returns the width a column should be based on it's breakpoints
  /// ex: col-lg-6 would set width to 50% of container width when breakpoint is lg
  static double columnWidthFromBreakPointTriggers({
    required double maxWidth,
    required double screenWidth,
    required List<String> breakPointTriggers,
  }) {
    // set values to unset
    Map<String, double> map = <String, double>{
      "": maxWidth,
      "sm": -1,
      "md": -1,
      "lg": -1,
      "xl": -1,
      "xxl": -1,
    };

    // set explicit values
    for (int i = 0; i < breakPointTriggers.length; i++) {
      if (RegExp(r"^col$").firstMatch(breakPointTriggers[i]) != null) {
        map[""] = (1 / 12) * maxWidth;
      } else if (RegExp(r"^col-(([0-9])|([0-9][0-2]))$")
              .firstMatch(breakPointTriggers[i]) !=
          null) {
        map[""] =
            (_getSuffixNumbersValue(breakPointTriggers[i]) / 12) * maxWidth;
      } else {
        for (int r = 1; r < _breakPointLabels.length; r++) {
          if (RegExp("^col-${_breakPointLabels[r]}-(([0-9])|([0-9][0-2]))\$")
                  .firstMatch(breakPointTriggers[i]) !=
              null) {
            map[_breakPointLabels[r]] =
                (_getSuffixNumbersValue(breakPointTriggers[i]) / 12) * maxWidth;
          }
        }
      }
    }

    // set inferred values
    for (int i = 1; i < _breakPointLabels.length; i++) {
      if (map[_breakPointLabels[i]] == -1) {
        map[_breakPointLabels[i]] = map[_breakPointLabels[i - 1]]!;
      }
    }

    return valueBasedOnBreakPointFromDefinedWidth(width: screenWidth, map: map);
  }

  /// Returns the size a text type should be based on the screen size.
  /// Can have a custom small and or large size passed, otherwise the
  /// default large and small sizes are used.
  /// The valid textType are:
  /// p
  /// h1
  /// h2
  /// h3
  /// h4
  /// h5
  /// h5
  static double getTextFontSize(
    BuildContext context,
    String textType, {
    double small = -1,
    double large = -1,
  }) {
    // confirm passed text type is valid
    if (!_textTypes.contains(textType)) {
      throw FormatException(
          "BSBreakPoints:\nMust pass a valid textType. Valid text types (case sensitive):\n${textType.toString()}\nYou passed:\n$textType");
    }

    // confirm a valid small and large is passed
    if (small < -1) {
      throw FormatException(
          "BSBreakPoints:\nThe passed value for small must be >= -1, you passed:\n${small.toString()}");
    }
    if (large < -1) {
      throw FormatException(
          "BSBreakPoints:\nThe passed value for large must be >= -1, you passed:\n${large.toString()}");
    }

    // set small and large values to default if not explicitly set
    if (small == -1) {
      small = _textTypesAndSizes[textType]!["sm"]!;
    }
    if (large == -1) {
      large = _textTypesAndSizes[textType]!["lg"]!;
    }

    // return size based on current screen size
    return valueBasedOnBreakPoint(
      context: context,
      map: {
        "": small,
        "sm": small,
        "md": large,
        "lg": large,
        "xl": large,
        "xxl": large,
      },
    );
  }
}
